# # from fiatlight import FunctionsCompositionGraph
# from imgui_bundle import immapp, imgui, ImVec2, ImVec4
# from typing import Any
#
#
# class FiatLux:
#     def __init__(self) -> None:
#         self._functions_composition_graph = None
#
#     def set_functions_composition_graph(self, functions_composition_graph: Any) -> None:
#         self._functions_composition_graph = functions_composition_graph
#
#     def draw(self) -> None:
#         imgui.push_id(str(id(self)))
#         imgui.pop_id()
